package com.autoussd.autoussdflutter

import android.util.Log
import androidx.activity.ComponentActivity
import androidx.annotation.NonNull
import com.autoussd.AutoUssd
import com.autoussd.models.Result

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result as MethodChannelResult

class AutoUssdFlutterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    companion object {
        private const val TAG = "AutoUssdFlutterPlugin"
        private const val CHANNEL_NAME = "autoussdflutter"
        private const val SESSION_COUNT_CALLBACK_NAME = "onSessionCount"
        private const val SESSION_RESULT_CALLBACK_NAME = "onSessionCallback"
        private const val SDK_NOT_INITIALIZED_ERROR_CODE = "SDK_NOT_INITIALIZED"
        private const val INVALID_SESSION_ERROR_CODE = "INVALID_SESSION"
    }

    private lateinit var channel: MethodChannel
    private var sdk: AutoUssd? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        Log.d(TAG, "On Attached To Engine")

        channel = MethodChannel(
            flutterPluginBinding.binaryMessenger,
            CHANNEL_NAME
        )

        channel.setMethodCallHandler(this)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        Log.d(TAG, "On Attached To Activity")

        try {
            sdk = AutoUssd(binding.activity as ComponentActivity, object : Result.Callback {
                override fun onSessionCount(count: Int) {
                    /*
                    * When the session count changes,
                    * notify the Flutter plugin via the method channel
                    */
                    binding.activity.runOnUiThread {
                        channel.invokeMethod(
                            SESSION_COUNT_CALLBACK_NAME,
                            count
                        )
                    }
                }

                override fun onSessionResult(result: Result) {
                    /*
                    * When a session execution result is ready,
                    * notify the Flutter plugin via the method channel.
                    *
                    * The execution result object is sent back as an array
                    * where the array elements are in the order of:
                    * [
                    * 	Session Id	: String
                    * 	Status		: String
                    * 	Description	: String
                    * 	Last Content: String?
                    * 	Parser		: String?
                    * ]
                    */
                    binding.activity.runOnUiThread {
                        channel.invokeMethod(
                            SESSION_RESULT_CALLBACK_NAME,
                            listOf(
                                result.sessionId,
                                result.status.toString(),
                                result.description,
                                result.lastContent,
                                result.parser,
                            )
                        )
                    }
                }
            })

            Log.d(TAG, "SDK initialized")
        } catch (err: Exception) {
            err.printStackTrace()
            Log.e(TAG, err.message ?: "Could not create instance of AutoUssd")
        }
    }

    override fun onDetachedFromActivity() {
        Log.d(TAG, "On Detached From Activity")
        sdk?.dispose()
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        Log.d(TAG, "On Reattached To Activity For Config Changes")
    }

    override fun onDetachedFromActivityForConfigChanges() {
        Log.d(TAG, "On Detached To Activity For Config Changes")
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        Log.d(TAG, "On Detached From Engine")
        channel.setMethodCallHandler(null)
        sdk?.dispose()
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannelResult) {
        Log.d(TAG, "On Method Call")

        if (call.method == "executeSession") {
            if (sdk == null) {
                Log.e(TAG, SDK_NOT_INITIALIZED_ERROR_CODE)

                result.error(
                    SDK_NOT_INITIALIZED_ERROR_CODE,
                    null,
                    null
                )
            } else {
                val sessionId: String? = call.argument<String>("sessionId")
                val variables: List<String>? = call.argument<List<String>>("variables")
                Log.d(TAG, "Variables: ${variables?.size ?: ""}")

                if (sessionId != null) {
                    sdk!!.executeSession(sessionId)
                } else {
                    Log.e(TAG, INVALID_SESSION_ERROR_CODE)

                    result.error(
                        INVALID_SESSION_ERROR_CODE,
                        "Session ID is null",
                        null
                    )
                }
            }
        } else result.notImplemented()
    }
}
