import 'dart:async';

import 'package:flutter/services.dart';

typedef _SessionCountCallback = void Function(int count);

typedef _SessionExecutionCallback = void Function(Result result);

class AutoUssdFlutter {
  static const String _CHANNEL_NAME = "autoussdflutter";
  static const String _EXECUTE_SESSION_METHOD_NAME = "executeSession";
  static const String _SESSION_COUNT_CALLBACK_NAME = "onSessionCount";
  static const String _SESSION_RESULT_CALLBACK_NAME = "onSessionCallback";
  static const MethodChannel _CHANNEL = const MethodChannel(
    _CHANNEL_NAME,
  );

  final _SessionCountCallback _onSessionCount;

  final _SessionExecutionCallback _onSessionExecuted;

  /// Primary constructor of the library.
  ///
  /// USSD session execution is asynchronous
  /// on the native platform thus a callback
  /// must be specified to be notified of
  /// the result
  AutoUssdFlutter(
    this._onSessionCount,
    this._onSessionExecuted,
  ) {
    _CHANNEL.setMethodCallHandler((call) async {
      if (call.method == _SESSION_COUNT_CALLBACK_NAME) {
        final count = call.arguments as int;

        _onSessionCount(count);
      } else if (call.method == _SESSION_RESULT_CALLBACK_NAME) {
        final List<dynamic> args = call.arguments;
        final sessionId = args[0] as String;
        final status = args[1] as String;
        final description = args[2] as String;
        final lastContent = args[3] as String?;
        final parser = args[4] as String?;

        _onSessionExecuted(
          Result._(
            sessionId,
            status,
            description,
            lastContent,
            parser,
          ),
        );
      }
    });
  }

  /// Executes a session matching
  /// the specified in your app.
  Future<void> executeSession(
    String sessionId, [
    List<String>? variables,
  ]) async {
    final args = {
      "sessionId": sessionId,
      "variables": variables,
    };

    await _CHANNEL.invokeMethod(
      _EXECUTE_SESSION_METHOD_NAME,
      args,
    );
  }
}

/// Represents the result
/// of a session execution
///
/// After the SDK executes a session,
/// it asynchronously returns an object
/// of this type which contains information
/// on what went on during the execution
class Result {
  late final String sessionId;
  late final ResultStatus status;
  late final String description;
  late final String? lastContent;
  late final String? parser;

  Result._(
    this.sessionId,
    String status,
    this.description,
    this.lastContent,
    this.parser,
  ) {
    this.status = Result._parseStatus(
      status,
    );
  }

  static ResultStatus _parseStatus(String status) {
    switch (status) {
      case "COMPLETED":
        return ResultStatus.COMPLETED;
      case "PARSED":
        return ResultStatus.PARSED;
      case "INVALID_SESSION":
        return ResultStatus.INVALID_SESSION;
      case "SESSION_TIMEOUT":
        return ResultStatus.SESSION_TIMEOUT;
      case "MENU_CONTENT_MISMATCH":
        return ResultStatus.MENU_CONTENT_MISMATCH;
      case "ACCOUNT_SUBSCRIPTION_EXPIRED":
        return ResultStatus.ACCOUNT_SUBSCRIPTION_EXPIRED;
      case "UNKNOWN_ERROR":
        return ResultStatus.UNKNOWN_ERROR;
      default:
        throw "Unknown status: $status";
    }
  }
}

/// Enum constants for report statuses
enum ResultStatus {
  /// Indicates that a session (without SMS parsers)
  /// executed successfully
  COMPLETED,

  /// Indicates that a session execution pending
  /// SMS confirmation has been parsed
  PARSED,

  /// Indicates that the session does
  /// not exist in app's sessions
  INVALID_SESSION,

  /// Indicates that the session
  /// timed-out during execution
  SESSION_TIMEOUT,

  /// Indicates that the menu match text didn't
  /// match that of the USSD dialog content
  MENU_CONTENT_MISMATCH,

  /// Indicates that the session owner's
  /// subscription period has elapsed
  ACCOUNT_SUBSCRIPTION_EXPIRED,

  /// Indicates that an unknown error occurred
  UNKNOWN_ERROR,
}
